from setuptools import setup, find_packages

setup(
    name="telona",
    version="0.1.0",
    description="Python SDK for Telona Inspector",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.8",
    install_requires=["requests>=2.28.0"],
    extras_require={
        "langchain": ["langchain>=0.1.0"],
        "llamaindex": ["llama-index>=0.10.0"],
    },
)
